import { __rest } from "tslib";
import { Injectable } from '@angular/core';
import { interpret, State, } from 'xstate';
import { filter, shareReplay, takeUntil } from 'rxjs/operators';
import { from, ReplaySubject } from 'rxjs';
export class XstateAngular {
    constructor() {
        this.unsubscribeSubject$ = new ReplaySubject(1);
        this.unsubscribe$ = this.unsubscribeSubject$.asObservable();
    }
    useMachine(machine, options = {}) {
        const { context, guards, actions, activities, services, delays, state: rehydratedState } = options, interpreterOptions = __rest(options, ["context", "guards", "actions", "activities", "services", "delays", "state"]);
        const machineConfig = {
            context,
            guards,
            actions,
            activities,
            services,
            delays,
        };
        const createdMachine = machine.withConfig(machineConfig, Object.assign(Object.assign({}, machine.context), context));
        this.service = interpret(createdMachine, interpreterOptions).start(rehydratedState ? State.create(rehydratedState) : undefined);
        const state$ = from(this.service).pipe(filter(({ changed, event }) => changed ||
            (changed === undefined && !!rehydratedState) ||
            (changed === undefined && !!((event === null || event === void 0 ? void 0 : event.type) === 'xstate.init'))), shareReplay(1), takeUntil(this.unsubscribe$));
        return { state$, send: this.service.send, service: this.service };
    }
    ngOnDestroy() {
        this.service.stop();
        this.unsubscribeSubject$.next();
    }
}
XstateAngular.decorators = [
    { type: Injectable }
];
//# sourceMappingURL=useMachine.js.map